header {
    import antlr.collections.AST;
}

/**
 * Example of a grammar which isn't LL(k) for any k.
 * Such grammars must be handled by syntactic predicates in Antlr.
 */
class NonLLkParser extends Parser;

options {
    exportVocab = NonLLk;
}

{ 
//     public static <P extends antlr.LLkParser, L extends antlr.CharScanner> 
//     void runParser(String args[]) 
//        throws Exception {
//         final P parser = new P(new L(System.in));
//         parser.startRule();
//         final AST t = parser.getAST();
//         System.out.println(t.toStringList());
//     }

    public static void main(String args[]) throws Exception {
        final NonLLkParser parser = 
            new NonLLkParser(new SimpleScanner(System.in));
        parser.exp();
        final AST t = parser.getAST();
        System.out.println(t.toStringList());
    }
}

exp     : qcAlt EOF
        ;
// Another example, variant of the question-colon operator with the
// dangling-else problem. The problem is that the ":" (aka "else") is both
// in the first set of (":" stmt) and its follow set.
// Solutions:
// 1. Distuingish the two branches by a syntactic predicate.
// 2. Checking if the generated parser choses the preferred alternative (most
//    closely nested rule), and in that case, turn off the ambiguity warning.
//    To match the first alternative is the default in Antlr, and the warning
//    can be turned off by
//      (options {greedy=true;} : ":" stmt)*

condition   : "true" 
            | "false"
            ;
qcAlt       : condition "?" stmt ((":" stmt) => ":" stmt)?
            ;
stmt        : qcAlt 
            ;
